"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
var isDevelopment = exports.isDevelopment = function isDevelopment() {
  return process.env.NODE_ENV !== "production";
};

var isFunction = function isFunction(f) {
  return typeof f === "function";
};

var toArray = exports.toArray = function toArray(field) {
  if (Array.isArray(field)) {
    return field;
  } else {
    return [field];
  }
};

var toError = exports.toError = function toError(message) {
  if (isDevelopment()) {
    throw new ReferenceError(message);
  } else {
    console.error(message);
  }
};

var validateFields = exports.validateFields = function validateFields(action, fetchFields) {
  if (!fetchFields) {
    toError("validateFields requires fetchFields function");
    return;
  }
  return function (params, _ref) {
    var properties = _ref.properties;

    var relFields = isFunction(fetchFields) ? toArray(fetchFields(params)) : toArray(fetchFields);
    relFields.filter(function (field) {
      return properties && properties[field] === undefined;
    }).forEach(function (field) {
      return toError("Field  \"" + field + "\" is missing from schema on \"" + action + "\"");
    });
  };
};

var fetchSchema = function fetchSchema(ref, schema) {
  if (ref.startsWith("#/")) {
    ref.substr(2).split("/");
    return ref.substr(2).split("/").reduce(function (schema, field) {
      return schema[field];
    }, schema);
  } else {
    toError("Only local references supported at this point");
    return undefined;
  }
};

var toRefField = function toRefField(field, _ref2) {
  var properties = _ref2.properties;

  if (properties[field]) {
    if (properties[field]["$ref"]) {
      return properties[field]["$ref"];
    } else if (properties[field].items && properties[field].items["$ref"]) {
      return properties[field].items["$ref"];
    }
  }
  return undefined;
};

var findRelSchema = exports.findRelSchema = function findRelSchema(field, schema) {
  var separator = field.indexOf(".");
  if (separator === -1) {
    var ref = toRefField(field, schema);
    return ref ? fetchSchema(ref, schema) : schema;
  } else {
    var parentField = field.substr(0, separator);
    var _ref3 = toRefField(parentField, schema);
    if (_ref3) {
      var refSchema = fetchSchema(_ref3, schema);
      return refSchema ? findRelSchema(field.substr(separator + 1), refSchema) : schema;
    } else {
      toError("Failed to find " + _ref3);
      return schema;
    }
  }
};